#ifndef _AKAITYPES_H
#define _AKAITYPES_H

#include "diskformats.h"

/* all d's have to be arrays of unsigned chars 
   used types in akaifs are: dw (4 bytes), tb (3 bytes), w (2 bytes) */

#define MAXNUMPARTITIONS 17

#define MAXPARSIZE_SECTORS 7680

#define akai_gettb(d) (d[0] + (d[1] << 8) + (d[2] << 16))

#define MIN(x,y) ((x) < (y) ? (x) : (y))

// The maximum blocksize which can be used natively, bounded by
// the AKAI_BLOCKSIZE
#define USE_BLOCKSIZE (MIN(PAGE_SIZE, AKAI_BLOCKSIZE))

// How many pages fit into one AKAI_BLOCK?
#define BLKMUL ((int)(AKAI_BLOCKSIZE / USE_BLOCKSIZE))

// Calculate the native physical blocknumber belonging to
// AKAIBLOCK x
#define REALBLOCK(x) ((x) * BLKMUL)

unsigned char * akaii2ascii(unsigned char *, int);
unsigned char * ascii2akaii(unsigned char *, int);

/* superblock private data structure */

struct akaifs_layout
{
  int number;                    // Number of partitions
  int offsets[MAXNUMPARTITIONS]; // Offsets relative to filesystem
};

/* akai inode types */

#define TYPE_PARTITION 3
#define TYPE_VOLUME    1
#define TYPE_FILE      2
#define TYPE_ROOT      0

#define GET_TYPE(ino)            (ino & 0x3)
#define GET_SOURCEPARTITION(ino) ((ino & 0xFC) >> 2)
#define GET_VOLINDEX(ino)         ((ino & 0xFF00) >> 8)
#define GET_FILEINDEX(ino)         ((ino & 0xFF0000) >> 16)


#define CONSTRUCT(o,b,p,t) \
  (t + ((p & 0x1F) << 2) + ((b & 0xFF) << 8) + ((o & 0xFF) << 16))

#endif
