#ifndef diskformats_h
#define diskformats_h

#include <linux/types.h>

#define AKAI_BLOCKSIZE 8192

struct s1000_partitionentry /* a partition entry is a reference to a volume! */
{
  u8 name[12]; /* The Volume name in akai ascii. */
  u8 type;
  u8 unknown; /* seems to be always zero? */
  u16 start; /* The offset of the volume, relative to the
			     partition, ie. partition_start + akai_getw(start)
			  */
};

struct s1000_partheader
{
  u16 size; /* This is the size, in 8192 sized blocks.  */
  u8 unknown[200];
  struct s1000_partitionentry volumes[100];
};

#define AFAT_EOF     0xC000
#define AFAT_SPECIAL 0x4000
#define AFAT_EMPTY   0x0000

#define AFAT_DATA(x) \
 ((x != AFAT_EOF) && (x != AFAT_SPECIAL) && (x != AFAT_EMPTY) && (x != -1))

/* The first sector of each *volume* contains directory entries, ie. the 
   contents of the volume. The size of each entry is 24. 8192/24 is arond
   344 entries. Not a nice round number. The question is: how many entries
   are present?

   I found some kind of 'end marker', "47 D7" on position 191,0 (that is
   byte 3056 of the sector). Is this the boundary of the file table?

   It makes sense: 128*24 = 3072. Are there only 127 places? I take this
   assumption. I should verify cq. test this with my sampler!
*/

#define MAX_DIRENTRIES 127

struct s1000_direntry
{
  unsigned char filename[12]; /* Filename */
  unsigned char extra[4]; 
  unsigned char filetype; /* 's' = sample, 'p' = program, etc. */
  unsigned char filelength[3]; /* File length in bytes */
  u16 startblock; /* Starting block on disk */
  unsigned char someid[2]; 
};

struct s3000_direntry
{
  unsigned char filename[12]; /* Filename */
  unsigned char extra[4];  /* 0,0,0,2 (1 for low density?) */
  unsigned char filetype; /* 211 = sample, 208 = program, etc. */
  unsigned char filelength[3]; /* File length in bytes */
  u16 startblock; /* Starting block on disk */
  unsigned char s3000id[2];
}; /* 24 bytes */

/*
  the FAT table.

  There seems to be a FAT table for every partition. 

  It starts at byte 1802; line 112, byte 8 of the akaish sectordump, right
  after the last partitionentry.

  Since all partitions on my disks have offset 3 for their first volume,
  I assume sector 0, 1 and 2 are for FAT and partition indexing.

  this means that the FAT size is 248 + 4096 + 2 * 8192 = 20728 bytes.

  Since startblocks in the direntries and partitionentries are 2 bytes,
  i assume the fat table contains 16-bit numbers. Eg 10364 sectors can be
  mapped.

  That's 84,901,888 megabyte. More than a partition (60 MB according to
  the sampler) could handle.

  Why?
  
  From some docs about FAT entries:

  For example, if a file starts at block 5, map entry 5 may contain the
  value 6, indicating that the next block to read is block 6. The next
  block to read is then given by map entry 6. The map entry for the last
  block of a file contains the value 32768. Unused blocks have a map entry
  of zero. 

  The first 4 blocks are needed for the file entries and map.

  64 * 24 byte long entries = 1536 ranging from 0 to 1535.

  1536 to 3136 (of 4736 voor hd) is the AKAI 'FAT'.
*/

#endif
